@@ -69,7 +69,6 @@ from . import settings_manager
 from . import speech
 from .input_event import BrailleEvent
 from .input_event import KeyboardEvent
-from .orca_i18n import ngettext
 
 _eventManager = event_manager.getManager()
 _scriptManager = script_manager.getManager()
@@ -245,8 +244,7 @@ def _processKeyboardEvent(event):
         if keyboardEvent.event_string == "F1":
             orca_state.learnModeEnabled = False
             return helpForOrca()
-        if keyboardEvent.event_string in ["1", "2"]:
-            orca_state.learnModeEnabled = False
+        if isPressedEvent and keyboardEvent.event_string in ["F2", "F3"]:
             return script.listOrcaShortcuts(keyboardEvent)
     if orca_state.capturingKeys:
         return False
@@ -626,7 +624,7 @@ def timeout(signum=None, frame=None):
     debug.println(debug.LEVEL_SEVERE,
                   "TIMEOUT: something has hung.  Aborting.")
     debug.printStack(debug.LEVEL_ALL)
-    examineProcesses()
+    debug.examineProcesses()
     die(EXIT_CODE_HANG)
 
 def shutdown(script=None, inputEvent=None):
@@ -651,8 +649,8 @@ def shutdown(script=None, inputEvent=None):
 
     orca_state.activeScript.presentMessage(messages.STOP_ORCA)
 
-    _eventManager.deactivate()
     _scriptManager.deactivate()
+    _eventManager.deactivate()
 
     # Shutdown all the other support.
     #
@@ -724,34 +722,6 @@ def abortOnSignal(signum, frame):
                   % signum)
     die(signum)
 
-def getCmdline(pid):
-    try:
-        openFile = os.popen('cat /proc/%s/cmdline' % pid)
-        cmdline = openFile.read()
-        openFile.close()
-    except:
-        cmdline = '(Could not obtain cmdline)'
-    cmdline = cmdline.replace('\x00', ' ')
-
-    return cmdline
-
-def examineProcesses():
-    desktop = pyatspi.Registry.getDesktop(0)
-    debug.println(
-        debug.LEVEL_ALL, 'INFO: Desktop has %i apps:' % desktop.childCount)
-    for i, app in enumerate(desktop):
-        pid = app.get_process_id()
-        cmd = getCmdline(pid)
-        try:
-            name = app.name
-        except:
-            name = 'ERROR: Could not get name'
-        else:
-            if name == '':
-                name = 'WARNING: Possible hang'
-        debug.println(
-            debug.LEVEL_ALL, '%3i. %s (pid: %s) %s' % (i+1, name, pid, cmd))
-
 def main():
     """The main entry point for Orca.  The exit codes for Orca will
     loosely be based on signals, where the exit code will be the
@@ -780,7 +750,9 @@ def main():
         message = messages.START_ORCA
         if not _settingsManager.getSetting('onlySpeakDisplayedText'):
             speech.speak(message, settings.voices.get(settings.SYSTEM_VOICE))
-        braille.displayMessage(message)
+        if _settingsManager.getSetting('enableBraille') \
+           or _settingsManager.getSetting('enableBrailleMonitor'):
+            braille.displayMessage(message)
     except:
         debug.printException(debug.LEVEL_SEVERE)
 
