@@ -437,10 +437,10 @@ class Script:
         if not cachedEvent or cachedEvent == event:
             return False
 
-        focus    = ["focus:", "object:state-changed:focused"]
+        focus    = ["object:state-changed:focused"]
         typing   = ["object:text-changed:insert", "object:text-changed:delete"]
         arrowing = ["object:text-caret-moved", "object:text-selection-changed",
-                    "object:selection-changed"]
+                    "object:selection-changed", "object:active-descendant-changed"]
 
         skip = False
         if (event.type in arrowing or event.type in typing) \
