@@ -25,20 +25,19 @@ __date__      = "$Date$"
 __copyright__ = "Copyright (c) 2010 Informal Informatica LTDA."
 __license__   = "LGPL"
 
-import orca.scripts.default as default
+import orca.scripts.toolkits.GAIL as GAIL
 import pyatspi
-from .script_utilities import Utilities
 
 ########################################################################
 #                                                                      #
 # The Eclipse script class.                                            #
 #                                                                      #
 ########################################################################
-class Script(default.Script):
+class Script(GAIL.Script):
 
     def __init__(self, app):
         """Creates a new script for the given application."""
-        default.Script.__init__(self, app)
+        GAIL.Script.__init__(self, app)
         self.movementKeys = ["Up", "Down", "Left", "Right", "Page_Up",
                    "Page_Down", "Home", "End"]
 
@@ -47,20 +46,17 @@ class Script(default.Script):
         otherObj. Overridden here so that we can give more feedback to user.
         """
 
-        if self.utilities.isDuplicateEvent(event):
-            return
-
         # Let the default script's normal behavior do its thing
         #
-        default.Script._presentTextAtNewCaretPosition(self, event, otherObj)
+        GAIL.Script._presentTextAtNewCaretPosition(self, event, otherObj)
 
         # check if the obj was spoken in the default script
         lastKey, mods = self.utilities.lastKeyAndModifiers()
         if lastKey in self.movementKeys:
             # already spoken in default script
             return
-        obj = otherObj or event.source
 
+        obj = otherObj or event.source
         if obj.getState().contains(pyatspi.STATE_SINGLE_LINE):
             return
 
@@ -79,26 +75,6 @@ class Script(default.Script):
 
         self._saveLastTextPosition(obj)
 
-    def onFocus(self, event):
-        """Called whenever an object gets focus.  Overridden here so that
-        so that we can avoid speaking text when caret moves after new text
-        receives focus and is spoken.
-
-        Arguments:
-        - event: the Event
-        """
-
-        # Let the default script's normal behavior do its thing
-        #
-        default.Script.onFocus(self, event)
-
-        self._saveLastTextPosition(event.source)
-
-    def getUtilities(self):
-        """Returns the utilities for this script."""
-
-        return Utilities(self)
-
     def onTextInserted(self, event):
         """Called whenever text is inserted into an object. Overridden here
         so that we can avoid speaking text when caret moves after new text
@@ -115,10 +91,8 @@ class Script(default.Script):
                 # or by commenting some block (ctrl+/).
                 # if not discarded, orca will speak all the text of the file.
                 return
-
             # Let the default script's normal behavior do its thing
-            #
-            default.Script.onTextInserted(self, event)
+            GAIL.Script.onTextInserted(self, event)
             self._saveLastTextPosition(event.source)
 
     def onTextDeleted(self, event):
@@ -132,7 +106,7 @@ class Script(default.Script):
 
         # Let the default script's normal behavior do its thing
         #
-        default.Script.onTextDeleted(self, event)
+        GAIL.Script.onTextDeleted(self, event)
         self._saveLastTextPosition(event.source)
 
     def _saveLastTextPosition(self, obj):
