@@ -74,22 +74,6 @@ class Script(default.Script):
 
         self.sayAllOnLoadCheckButton = None
 
-    def getListeners(self):
-        """Sets up the AT-SPI event listeners for this script."""
-
-        listeners = default.Script.getListeners(self)
-
-        listeners["document:reload"]                        = \
-            self.onDocumentReload
-        listeners["document:load-complete"]                 = \
-            self.onDocumentLoadComplete
-        listeners["document:load-stopped"]                  = \
-            self.onDocumentLoadStopped
-        listeners["object:state-changed:busy"]              = \
-            self.onStateChanged
-
-        return listeners
-
     def setupInputEventHandlers(self):
         """Defines InputEventHandler fields for this script that can be
         called by the key and braille bindings."""
@@ -227,13 +211,13 @@ class Script(default.Script):
         default.Script.onCaretMoved(self, event)
 
     def onDocumentReload(self, event):
-        """Called when the reload button is hit for a web page."""
+        """Callback for document:reload accessibility events."""
 
         if event.source.getRole() == pyatspi.ROLE_DOCUMENT_FRAME:
             self._loadingDocumentContent = True
 
     def onDocumentLoadComplete(self, event):
-        """Called when a web page load is completed."""
+        """Callback for document:load-complete accessibility events."""
 
         if event.source.getRole() != pyatspi.ROLE_DOCUMENT_FRAME:
             return
@@ -254,17 +238,13 @@ class Script(default.Script):
             self.sayAll(None)
 
     def onDocumentLoadStopped(self, event):
-        """Called when a web page load is interrupted."""
+        """Callback for document:load-stopped accessibility events."""
 
         if event.source.getRole() == pyatspi.ROLE_DOCUMENT_FRAME:
             self._loadingDocumentContent = False
 
-    def onFocus(self, event):
-        """Called whenever an object gets focus.
-
-        Arguments:
-        - event: the Event
-        """
+    def onFocusedChanged(self, event):
+        """Callback for object:state-changed:focused accessibility events."""
 
         obj = event.source
         role = obj.getRole()
@@ -276,7 +256,7 @@ class Script(default.Script):
                        pyatspi.ROLE_LIST_ITEM,
                        pyatspi.ROLE_RADIO_BUTTON]
         if role in widgetRoles:
-            default.Script.onFocus(self, event)
+            default.Script.onFocusedChanged(self, event)
             return
 
         textRoles = [pyatspi.ROLE_HEADING,
@@ -292,29 +272,25 @@ class Script(default.Script):
             if lastKey in self.CARET_NAVIGATION_KEYS:
                 return
 
-        default.Script.onFocus(self, event)
+        default.Script.onFocusedChanged(self, event)
 
-    def onStateChanged(self, event):
-        """Called whenever an object's state changes.
+    def onBusyChanged(self, event):
+        """Callback for object:state-changed:busy accessibility events."""
 
-        Arguments:
-        - event: the Event
-        """
-
-        if not event.type.startswith("object:state-changed:busy"):
-            default.Script.onStateChanged(self, event)
+        obj = event.source
+        try:
+            role = obj.getRole()
+            name = obj.name
+        except:
             return
 
-        if not event.source \
-           or event.source.getRole() != pyatspi.ROLE_DOCUMENT_FRAME \
-           or not self._isBrowser:
+        if role != pyatspi.ROLE_DOCUMENT_FRAME or not self._isBrowser:
             return
 
         if event.detail1:
             self.presentMessage(messages.PAGE_LOADING_START)
-        elif event.source.name:
-            self.presentMessage(
-                messages.PAGE_LOADING_END_NAMED % event.source.name)
+        elif name:
+            self.presentMessage(messages.PAGE_LOADING_END_NAMED % name)
         else:
             self.presentMessage(messages.PAGE_LOADING_END)
 
@@ -599,11 +575,11 @@ class Script(default.Script):
 
         document = utils.findAncestor(
             obj, lambda x: x.getRole() == pyatspi.ROLE_DOCUMENT_FRAME)
-        if not document:
+        if not document or document.getState().contains(pyatspi.STATE_BUSY):
             return
 
         allTextObjs = utils.findAllDescendants(
-            document, lambda x: 'Text' in utils.listInterfaces(x))
+            document, lambda x: x and 'Text' in utils.listInterfaces(x))
         allTextObjs = allTextObjs[allTextObjs.index(obj):len(allTextObjs)]
         textObjs = [x for x in allTextObjs if x.parent not in allTextObjs]
         if not textObjs:
